@@ -17,6 +17,7 @@
 package collector
 
 import (
+	"context"
 	"errors"
 	"fmt"
 	"math"
@@ -26,7 +27,7 @@ import (
 	"sync"
 	"time"
 
-	"github.com/coreos/go-systemd/dbus"
+	"github.com/coreos/go-systemd/v22/dbus"
 	"github.com/go-kit/log"
 	"github.com/go-kit/log/level"
 	"github.com/prometheus/client_golang/prometheus"
@@ -57,6 +58,8 @@ var (
 	enableTaskMetrics      = kingpin.Flag("collector.systemd.enable-task-metrics", "Enables service unit tasks metrics unit_tasks_current and unit_tasks_max").Bool()
 	enableRestartsMetrics  = kingpin.Flag("collector.systemd.enable-restarts-metrics", "Enables service unit metric service_restart_total").Bool()
 	enableStartTimeMetrics = kingpin.Flag("collector.systemd.enable-start-time-metrics", "Enables service unit metric unit_start_time_seconds").Bool()
+
+	systemdVersionRE = regexp.MustCompile(`[0-9]{3,}(\.[0-9]+)?`)
 )
 
 type systemdCollector struct {
@@ -72,7 +75,6 @@ type systemdCollector struct {
 	socketCurrentConnectionsDesc  *prometheus.Desc
 	socketRefusedConnectionsDesc  *prometheus.Desc
 	systemdVersionDesc            *prometheus.Desc
-	systemdVersion                int
 	unitIncludePattern            *regexp.Regexp
 	unitExcludePattern            *regexp.Regexp
 	logger                        log.Logger
@@ -129,7 +131,7 @@ func NewSystemdCollector(logger log.Logger) (Collector, error) {
 		"Total number of refused socket connections", []string{"name"}, nil)
 	systemdVersionDesc := prometheus.NewDesc(
 		prometheus.BuildFQName(namespace, subsystem, "version"),
-		"Detected systemd version", []string{}, nil)
+		"Detected systemd version", []string{"version"}, nil)
 
 	if *oldUnitExclude != "" {
 		if !unitExcludeSet {
@@ -152,12 +154,6 @@ func NewSystemdCollector(logger log.Logger) (Collector, error) {
 	level.Info(logger).Log("msg", "Parsed flag --collector.systemd.unit-exclude", "flag", *unitExclude)
 	unitExcludePattern := regexp.MustCompile(fmt.Sprintf("^(?:%s)$", *unitExclude))
 
-	systemdVersion := getSystemdVersion(logger)
-	if systemdVersion < minSystemdVersionSystemState {
-		level.Warn(logger).Log("msg", "Detected systemd version is lower than minimum", "current", systemdVersion, "minimum", minSystemdVersionSystemState)
-		level.Warn(logger).Log("msg", "Some systemd state and timer metrics will not be available")
-	}
-
 	return &systemdCollector{
 		unitDesc:                      unitDesc,
 		unitStartTimeDesc:             unitStartTimeDesc,
@@ -171,7 +167,6 @@ func NewSystemdCollector(logger log.Logger) (Collector, error) {
 		socketCurrentConnectionsDesc:  socketCurrentConnectionsDesc,
 		socketRefusedConnectionsDesc:  socketRefusedConnectionsDesc,
 		systemdVersionDesc:            systemdVersionDesc,
-		systemdVersion:                systemdVersion,
 		unitIncludePattern:            unitIncludePattern,
 		unitExcludePattern:            unitExcludePattern,
 		logger:                        logger,
@@ -188,6 +183,17 @@ func (c *systemdCollector) Update(ch chan<- prometheus.Metric) error {
 	}
 	defer conn.Close()
 
+	systemdVersion, systemdVersionFull := c.getSystemdVersion(conn)
+	if systemdVersion < minSystemdVersionSystemState {
+		level.Debug(c.logger).Log("msg", "Detected systemd version is lower than minimum, some systemd state and timer metrics will not be available", "current", systemdVersion, "minimum", minSystemdVersionSystemState)
+	}
+	ch <- prometheus.MustNewConstMetric(
+		c.systemdVersionDesc,
+		prometheus.GaugeValue,
+		systemdVersion,
+		systemdVersionFull,
+	)
+
 	allUnits, err := c.getAllUnits(conn)
 	if err != nil {
 		return fmt.Errorf("couldn't get units: %w", err)
@@ -234,7 +240,7 @@ func (c *systemdCollector) Update(ch chan<- prometheus.Metric) error {
 		}()
 	}
 
-	if c.systemdVersion >= minSystemdVersionSystemState {
+	if systemdVersion >= minSystemdVersionSystemState {
 		wg.Add(1)
 		go func() {
 			defer wg.Done()
@@ -252,15 +258,12 @@ func (c *systemdCollector) Update(ch chan<- prometheus.Metric) error {
 		level.Debug(c.logger).Log("msg", "collectSockets took", "duration_seconds", time.Since(begin).Seconds())
 	}()
 
-	if c.systemdVersion >= minSystemdVersionSystemState {
+	if systemdVersion >= minSystemdVersionSystemState {
 		begin = time.Now()
 		err = c.collectSystemState(conn, ch)
 		level.Debug(c.logger).Log("msg", "collectSystemState took", "duration_seconds", time.Since(begin).Seconds())
 	}
 
-	ch <- prometheus.MustNewConstMetric(
-		c.systemdVersionDesc, prometheus.GaugeValue, float64(c.systemdVersion))
-
 	return err
 }
 
@@ -268,14 +271,14 @@ func (c *systemdCollector) collectUnitStatusMetrics(conn *dbus.Conn, ch chan<- p
 	for _, unit := range units {
 		serviceType := ""
 		if strings.HasSuffix(unit.Name, ".service") {
-			serviceTypeProperty, err := conn.GetUnitTypeProperty(unit.Name, "Service", "Type")
+			serviceTypeProperty, err := conn.GetUnitTypePropertyContext(context.TODO(), unit.Name, "Service", "Type")
 			if err != nil {
 				level.Debug(c.logger).Log("msg", "couldn't get unit type", "unit", unit.Name, "err", err)
 			} else {
 				serviceType = serviceTypeProperty.Value.Value().(string)
 			}
 		} else if strings.HasSuffix(unit.Name, ".mount") {
-			serviceTypeProperty, err := conn.GetUnitTypeProperty(unit.Name, "Mount", "Type")
+			serviceTypeProperty, err := conn.GetUnitTypePropertyContext(context.TODO(), unit.Name, "Mount", "Type")
 			if err != nil {
 				level.Debug(c.logger).Log("msg", "couldn't get unit type", "unit", unit.Name, "err", err)
 			} else {
@@ -293,7 +296,7 @@ func (c *systemdCollector) collectUnitStatusMetrics(conn *dbus.Conn, ch chan<- p
 		}
 		if *enableRestartsMetrics && strings.HasSuffix(unit.Name, ".service") {
 			// NRestarts wasn't added until systemd 235.
-			restartsCount, err := conn.GetUnitTypeProperty(unit.Name, "Service", "NRestarts")
+			restartsCount, err := conn.GetUnitTypePropertyContext(context.TODO(), unit.Name, "Service", "NRestarts")
 			if err != nil {
 				level.Debug(c.logger).Log("msg", "couldn't get unit NRestarts", "unit", unit.Name, "err", err)
 			} else {
@@ -311,7 +314,7 @@ func (c *systemdCollector) collectSockets(conn *dbus.Conn, ch chan<- prometheus.
 			continue
 		}
 
-		acceptedConnectionCount, err := conn.GetUnitTypeProperty(unit.Name, "Socket", "NAccepted")
+		acceptedConnectionCount, err := conn.GetUnitTypePropertyContext(context.TODO(), unit.Name, "Socket", "NAccepted")
 		if err != nil {
 			level.Debug(c.logger).Log("msg", "couldn't get unit NAccepted", "unit", unit.Name, "err", err)
 			continue
@@ -320,7 +323,7 @@ func (c *systemdCollector) collectSockets(conn *dbus.Conn, ch chan<- prometheus.
 			c.socketAcceptedConnectionsDesc, prometheus.CounterValue,
 			float64(acceptedConnectionCount.Value.Value().(uint32)), unit.Name)
 
-		currentConnectionCount, err := conn.GetUnitTypeProperty(unit.Name, "Socket", "NConnections")
+		currentConnectionCount, err := conn.GetUnitTypePropertyContext(context.TODO(), unit.Name, "Socket", "NConnections")
 		if err != nil {
 			level.Debug(c.logger).Log("msg", "couldn't get unit NConnections", "unit", unit.Name, "err", err)
 			continue
@@ -330,7 +333,7 @@ func (c *systemdCollector) collectSockets(conn *dbus.Conn, ch chan<- prometheus.
 			float64(currentConnectionCount.Value.Value().(uint32)), unit.Name)
 
 		// NRefused wasn't added until systemd 239.
-		refusedConnectionCount, err := conn.GetUnitTypeProperty(unit.Name, "Socket", "NRefused")
+		refusedConnectionCount, err := conn.GetUnitTypePropertyContext(context.TODO(), unit.Name, "Socket", "NRefused")
 		if err != nil {
 			//log.Debugf("couldn't get unit '%s' NRefused: %s", unit.Name, err)
 		} else {
@@ -348,7 +351,7 @@ func (c *systemdCollector) collectUnitStartTimeMetrics(conn *dbus.Conn, ch chan<
 		if unit.ActiveState != "active" {
 			startTimeUsec = 0
 		} else {
-			timestampValue, err := conn.GetUnitProperty(unit.Name, "ActiveEnterTimestamp")
+			timestampValue, err := conn.GetUnitPropertyContext(context.TODO(), unit.Name, "ActiveEnterTimestamp")
 			if err != nil {
 				level.Debug(c.logger).Log("msg", "couldn't get unit StartTimeUsec", "unit", unit.Name, "err", err)
 				continue
@@ -366,7 +369,7 @@ func (c *systemdCollector) collectUnitTasksMetrics(conn *dbus.Conn, ch chan<- pr
 	var val uint64
 	for _, unit := range units {
 		if strings.HasSuffix(unit.Name, ".service") {
-			tasksCurrentCount, err := conn.GetUnitTypeProperty(unit.Name, "Service", "TasksCurrent")
+			tasksCurrentCount, err := conn.GetUnitTypePropertyContext(context.TODO(), unit.Name, "Service", "TasksCurrent")
 			if err != nil {
 				level.Debug(c.logger).Log("msg", "couldn't get unit TasksCurrent", "unit", unit.Name, "err", err)
 			} else {
@@ -378,7 +381,7 @@ func (c *systemdCollector) collectUnitTasksMetrics(conn *dbus.Conn, ch chan<- pr
 						float64(val), unit.Name)
 				}
 			}
-			tasksMaxCount, err := conn.GetUnitTypeProperty(unit.Name, "Service", "TasksMax")
+			tasksMaxCount, err := conn.GetUnitTypePropertyContext(context.TODO(), unit.Name, "Service", "TasksMax")
 			if err != nil {
 				level.Debug(c.logger).Log("msg", "couldn't get unit TasksMax", "unit", unit.Name, "err", err)
 			} else {
@@ -400,7 +403,7 @@ func (c *systemdCollector) collectTimers(conn *dbus.Conn, ch chan<- prometheus.M
 			continue
 		}
 
-		lastTriggerValue, err := conn.GetUnitTypeProperty(unit.Name, "Timer", "LastTriggerUSec")
+		lastTriggerValue, err := conn.GetUnitTypePropertyContext(context.TODO(), unit.Name, "Timer", "LastTriggerUSec")
 		if err != nil {
 			level.Debug(c.logger).Log("msg", "couldn't get unit LastTriggerUSec", "unit", unit.Name, "err", err)
 			continue
@@ -434,9 +437,9 @@ func (c *systemdCollector) collectSystemState(conn *dbus.Conn, ch chan<- prometh
 
 func newSystemdDbusConn() (*dbus.Conn, error) {
 	if *systemdPrivate {
-		return dbus.NewSystemdConnection()
+		return dbus.NewSystemdConnectionContext(context.TODO())
 	}
-	return dbus.New()
+	return dbus.NewWithContext(context.TODO())
 }
 
 type unit struct {
@@ -444,7 +447,7 @@ type unit struct {
 }
 
 func (c *systemdCollector) getAllUnits(conn *dbus.Conn) ([]unit, error) {
-	allUnits, err := conn.ListUnits()
+	allUnits, err := conn.ListUnitsContext(context.TODO())
 	if err != nil {
 		return nil, err
 	}
@@ -488,24 +491,19 @@ func filterUnits(units []unit, includePattern, excludePattern *regexp.Regexp, lo
 	return filtered
 }
 
-func getSystemdVersion(logger log.Logger) int {
-	conn, err := newSystemdDbusConn()
-	if err != nil {
-		level.Warn(logger).Log("msg", "Unable to get systemd dbus connection, defaulting systemd version to 0", "err", err)
-		return 0
-	}
-	defer conn.Close()
+func (c *systemdCollector) getSystemdVersion(conn *dbus.Conn) (float64, string) {
 	version, err := conn.GetManagerProperty("Version")
 	if err != nil {
-		level.Warn(logger).Log("msg", "Unable to get systemd version property, defaulting to 0")
-		return 0
+		level.Debug(c.logger).Log("msg", "Unable to get systemd version property, defaulting to 0")
+		return 0, ""
 	}
-	re := regexp.MustCompile(`[0-9][0-9][0-9]`)
-	version = re.FindString(version)
-	v, err := strconv.Atoi(version)
+	version = strings.TrimPrefix(strings.TrimSuffix(version, `"`), `"`)
+	level.Debug(c.logger).Log("msg", "Got systemd version", "version", version)
+	parsedVersion := systemdVersionRE.FindString(version)
+	v, err := strconv.ParseFloat(parsedVersion, 64)
 	if err != nil {
-		level.Warn(logger).Log("msg", "Got invalid systemd version", "version", version)
-		return 0
+		level.Debug(c.logger).Log("msg", "Got invalid systemd version", "version", version)
+		return 0, ""
 	}
-	return v
+	return v, version
 }
